
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    display.h
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        Type definitions, prototypes, defines for Display 
 *                 class and closely related classes Font, Style, and
 *                 DisplayEvent.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/

#ifndef _DISPLAY
#define _DISPLAY

#ifndef _COLOR
#include "color.h"
#endif

/* A Coordinate is a signed short value
 * which allows for a document which is
 * 68 VGA dpys in height, which may
 * just be enough.
 */
#if 0
typedef short Coordinate;
typedef short Dimension;
#define MAX_DIMENSION (0xfffe)
#define MAX_COORD (0x7fff)
#define MIN_COORD (0x8001)
#define INVALID_DIMENSION (0xffff)
#define INVALID_COORDINATE (0x8000)
#else 
typedef long Coordinate;
typedef long Dimension;
#define MAX_DIMENSION (0xfffffffe)
#define MAX_COORD (0x7ffffffe)
#define MIN_COORD (0x80000001)
#define INVALID_DIMENSION  (0xffffffff)
#define INVALID_COORDINATE (0x80000000)
#endif

#define MIN_DIMENSION (0)


typedef unsigned char FontSize;
#define INVALID_FONTSIZE (0xff)
#define MAX_FONTSIZE (250)



/* Because several types of displays are used, we need to
 * create an all-emcompassing event class to handle them all. 
 */
enum { 
	EVENT_NONE=0, EVENT_KEY, EVENT_MOUSE, EVENT_REFRESH,
	EVENT_LEFT, EVENT_RIGHT, EVENT_UP, EVENT_DOWN,
	EVENT_PGUP, EVENT_PGDN, EVENT_HOME, EVENT_END,
};
typedef struct _de {
	unsigned char type;
	Dimension x0, y0, x1, y1;
	unsigned short value;
}
DisplayEvent;



typedef struct _fd
{
	char *family;
	char *facename;
	char *longname;
	unsigned short total_chars;

	unsigned char size;
	unsigned int is_bold;
	unsigned int is_slanted;
	unsigned char max_width;

	unsigned char max_height;
	unsigned char ascent;
	unsigned char descent;

	unsigned char *bytes;
	unsigned char *widths;
	char *x_offsets;
	char *y_offsets;
	unsigned short *indices;
	char **non_ansi_names;
	struct _fd *next;
}
Font;



/* Attribute flags are intended to the or'able. */
#define ATTR_NONE 0
#define ATTR_BOLD 1
#define ATTR_ITALIC 2
#define ATTR_UNDERLINE 4
#define ATTR_LINK 8
#define ATTR_PLAIN 16
#define ATTR_SUPER 32
#define ATTR_SUB 64
#define ATTR_STRIKE 128
#define ATTR_OVERBAR 256



typedef struct _style 
{
	String name; /* if the Style has a name, this is it; else NULL */

	unsigned short flags;
	Dimension super_amount; /* won't ever be > 255 */
	Dimension interchar_spacing;  /* won't ever be > 65535 */
	Color fg;  
	Color bg;

	/* Font info */
	String family;
	FontSize size; /* no font is > 255 point */
	Font *fontdata;

	struct _style *next;
} 
Style;



#ifndef _IMAGE
#include "image.h"
#endif



typedef struct _dpy {
	Dimension width;  /* drawable area */
	Dimension height;

	Dimension actual_width;  /* complete area including scrollbars */
	Dimension actual_height;

	/* Defaults */
	Color fgcolor;
	Color bgcolor;
	Color highlight_color;
	Color pointer_fgcolor; /* mouse pointer, not text cursor */
	Color pointer_bordercolor;
	String family;
	FontSize fontsize;

	/* Assigned pixel values */
	unsigned char fg, bg;
	unsigned char white, black, red, green, blue;
	unsigned char grey1;
	unsigned char grey2;
	unsigned char grey3;
	unsigned char grey4;
	unsigned char grey5;
	unsigned char grey6;
	unsigned char grey7;
	unsigned char grey8;

	/* Scrollbar information */
	unsigned char vscroll_width;
	unsigned char hscroll_height;

	unsigned char vscroll_value;  /* current value 0..99 */
	unsigned short vscroll_jump;   /* NOT USED -- amount to increase by via arrow keys */

	unsigned char hscroll_value;  /* current value 0..99 */
	unsigned short hscroll_jump;   /* NOT USED -- amount to increase by via arrow keys */

	Dimension heading_space; /* Space before/after H1, H2, etc */
	Dimension horizontal_line_space; /* Space before/after HR */
	Dimension list_indentation_space;
	Dimension interline_padding;
	Dimension table_border_size;
	Dimension table_cell_spacing;
	Dimension table_cell_padding;
	Dimension interparagraph_space; /* usually the same as heading_space */

	void	(*refresh)(Coordinate,Coordinate,Coordinate,Coordinate,Coordinate,Coordinate); 
	void	(*resize)(Dimension,Dimension);

	void	(*clear)(void); /* does NOT clear scrollbar/etc areas */

	void	(*set_style)(Style*);

	int	(*set_font_complete)(String,FontSize,unsigned short);
	int	(*reset_font_complete)(void);
	int	(*set_font)(String family);
	int	(*set_size)(FontSize);
	int	(*set_attr)(unsigned short attr);
	Font*   (*get_font_metrics)(String,FontSize,unsigned short);

	void	(*set_colors)(Color, Color);
	void	(*reset_colors)(void);
	int	(*color_to_pixel)(String);
	int	(*rgb_to_pixel)(Color);

	void	(*set_clip)(Coordinate,Coordinate,Coordinate,Coordinate);
	void	(*reset_clip)(void);

	/* This should be a style.c routine! */
	void	(*text_dims)(String,Style*,Dimension*,Dimension*,Dimension*);
	Dimension	(*get_font_height)(void);

	void	(*draw_text)(Coordinate,Coordinate,String,unsigned char);
	int	(*convert_special_expr)(String);

	void	(*draw_rectangle)(Coordinate,Coordinate,Coordinate,Coordinate);
	void	(*draw_3d_rectangle)(Coordinate,Coordinate,Coordinate,Coordinate,
			Dimension border_size, Color,Color,Color,Color);
	void	(*filled_rectangle)(Coordinate,Coordinate,Coordinate,Coordinate);
	void	(*highlight_area)(Coordinate,Coordinate,Coordinate,Coordinate);
	void	(*unhighlight_area)(Coordinate,Coordinate,Coordinate,Coordinate);
	void	(*draw_x)(Coordinate,Coordinate,Coordinate,Coordinate);
	void	(*draw_hline)(Coordinate,Coordinate,Coordinate);
	void	(*draw_vline)(Coordinate,Coordinate,Coordinate);
	void	(*draw_line)(Coordinate,Coordinate,Coordinate,Coordinate);

	void	(*get_image)(Coordinate,Coordinate,Dimension,Dimension,Image*);
	void	(*put_image)(Coordinate,Coordinate,Image*);

	void	(*vscroll_enable)(void);
	void	(*hscroll_enable)(void);
	void	(*vscroll_disable)(void);
	void	(*hscroll_disable)(void);
	void	(*vscroll_move)(int);
	void	(*hscroll_move)(int);

	void	(*pointer_enable)(void);
	void	(*pointer_disable)(void);
	void	(*pointer_hide)(void);
	void	(*pointer_unhide)(void);

	void	(*draw_twoline_centered_popup)(String,String,Image*);

	int	(*get_event)(DisplayEvent*);

	void	(*dump)(void);

	void	(*close)(void);

	void	(*set_callback)(int,int (*ptr)(struct _dpy*, Coordinate, Coordinate, Coordinate, Coordinate)); /* if no buffering (e.g. VGA) */

	void	(*allocate_buffers)(Dimension,Dimension);

	void	(*scroll_up)(Dimension);
	void	(*scroll_down)(Dimension);
	void	(*scroll_right)(Dimension);
	void	(*scroll_left)(Dimension);

	void	(*restore_background)(Coordinate, Coordinate);

	/*------callbacks------*/
	int	(*refresh_callback)(struct _dpy*, Coordinate, Coordinate, Coordinate, Coordinate);
	int	(*resize_callback)(struct _dpy*, Coordinate, Coordinate, Coordinate, Coordinate);

}
DisplayDevice;


#define DISPLAY_NO_CALLBACK (0)
#define DISPLAY_REFRESH_CALLBACK (1)
#define DISPLAY_RESIZE_CALLBACK (2)
#define DISPLAY_MAX_CALLBACK (2)


extern void display_free (DisplayDevice*);
extern DisplayDevice *display_new (void);

extern Font* lookup_font (DisplayDevice*,String,FontSize,unsigned short);
extern Font* font_new(void);
extern void font_free(Font*);
extern void font_dump(FILE*,Font*);

extern Style *style_new ();
extern void style_reset (Style*);
extern Style *style_dup (Style*);
extern void style_copy (Style*, Style*);
extern void style_fix_font (DisplayDevice*, Style*);
extern void style_free (Style*);
extern void style_dump (FILE*,Style*);
extern void style_set_font (Style*, Font*);


#endif

