
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    image.h
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        Type definitions, prototypes, defines for Image class
 *                 and image file reading and writing.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/


#ifndef _IMAGE
#define _IMAGE

#ifndef _DISPLAY
#include "display.h"
#endif


typedef struct {
	Coordinate x,y;
	Dimension width, height;
	unsigned char *pixmap; /* Image is for 8-bit pixels only */
} Image;


extern Image *image_new (void);
extern void image_free (Image*);
extern void image_clear (Image*, unsigned char pixel);

extern void image_write_xbm (Image*, FILE*);
extern int image_read_xbm (Image*,FILE*,unsigned long,unsigned long);
extern void image_copy (Image *dest, 
	Coordinate dest_x, Coordinate dest_y, 
	Coordinate src_x, Coordinate src_y, 
	Dimension copy_w, Dimension copy_h, Image *src);
extern void image_init (Image *im, Dimension width, Dimension height);
extern void image_copy_from_xbm (Image *dest, Coordinate x, Coordinate y,
	Dimension w, Dimension h, unsigned char *xbm, unsigned long pixel);
extern void image_dump (FILE*,Image*);

extern void image_draw_pixel (Image *,Coordinate,Coordinate,unsigned char);
extern void image_draw_line (Image *,Coordinate,Coordinate,Coordinate,
	Coordinate,unsigned char);
extern void image_draw_rect (Image *,Coordinate,Coordinate,Coordinate,
	Coordinate,unsigned char);
extern void image_draw_filled_rect (Image *,Coordinate,Coordinate,Coordinate,
	Coordinate,unsigned char);
extern void image_draw_hline (Image *,Coordinate,Coordinate,Coordinate,
	unsigned char);
extern void image_draw_vline (Image *,Coordinate,Coordinate,Coordinate,
	unsigned char);
extern void image_draw_oval (Image *,Coordinate,Coordinate,Coordinate,
	Coordinate,unsigned char);
extern void image_draw_filled_oval (Image *,Coordinate,Coordinate,Coordinate,
	Coordinate,unsigned char);

extern void 
image_draw_char (Image *im, Font *fontdata,
		 int simulate_boldface,
		 int simulate_italic,
                 int ch, Coordinate x0, Coordinate y0,
		 unsigned char pixel);

extern void
image_draw_text (Image *im, Font *fontdata,
		 int simulate_boldface,
		 int simulate_italic,
                 String str,
		 Coordinate startx, Coordinate starty,
		 unsigned char pixel);

#endif





