
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    parse.c
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    26 Nov 2000
 * Purpose:        HTML Parser.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>


#include "defs.h"
#include "malloc.h"
#include "main.h"
#include "word.h"
#include "read.h"
#include "error.h"
#include "parse.h"
#include "special.h"


static Word* ungot_word=NULL;

void
word_unget (Word *w) {
	CHECK_PARAM_NOT_NULL(w);

	if (ungot_word)
		error_handler ("multiple ungot Words");

	ungot_word=w;
}

Word* 
word_read (FILE* f, int break_at_whitespace) {
	int ch;
	int ix=0;
#define MAX_WORD_LEN 10000
	char str[MAX_WORD_LEN];
	unsigned char got_whitespace=FALSE;
	unsigned char following_whitespace=0;
	Word *w;

	CHECK_PARAM_NOT_NULL(f);

	if (ungot_word) {
		w=ungot_word;
		ungot_word=NULL;
		return w;
	}

	/* Before we can get to the meaty part, 
	 * we must remove the fat.
	 */
	if (!break_at_whitespace) {
		ch=my_getchar(f);
	}
	else do {
		/* Skip spaces
		 */
		while (EOF!=(ch=my_getchar(f))) {
			if (ch!=' ')break;
			if (ch==' ')got_whitespace=TRUE;
		}
		if(got_whitespace)  { 
			my_unget(ch); 
			return word_new(" "); 
		}
		if(ch==EOF) return NULL;

		/* Skip comments
		 */
		while (ch=='<') {
			do { ch=my_getchar(f); } while (ch==' ');
			if (ch=='!') {
				int counter=1;
				while(EOF!=(ch=my_getchar(f))) {
					/* This is legal: <! <hr> <a> > */
					if (ch=='>') counter--;
					if (ch=='<') counter++;
					if (!counter) break;
				}
				ch=my_getchar(f);
			} else {
				my_unget(ch);
				ch='<';
				break;
			}
			ix=0;
		}
		if(ch==EOF) return NULL;
	} while (ch == ' ');

	/* Read a tag...
	 */
	if(ch=='<') {
		int in_str=0, quote;
		str[ix++]=ch;
		while (EOF!=(ch=my_getchar(f))) {
			if(in_str && ch==quote) { in_str=FALSE; }
			else if(ch=='"'||ch=='\'') { 
				/* We're entering a string. If there is a
				 * string within a string e.g. 'name="Dick" '
				 * then we need to preserve the outer quote
				 * char. 
				 */
				if (!in_str) {
					in_str=TRUE; quote=ch;
				}
			}
			str[ix++]=in_str ? ch : tolower(ch);
			if (!in_str && ch=='>')break;
			if (ix==MAX_WORD_LEN) {
str[ix]=0;
fprintf (logfile,"0str=%s\n", str);
				error_handler ("encountered an extremely long word");
			}
		}
		str[ix]=0;
		/* Replace empty tags with whitespace */
		if(ix==2 && str[0]=='<' && str[1]=='>') {
			ix=1;
			str[0]=' ';
		}
	}
	else if(ch=='&') {
		str[ix++]=ch;
		ch=my_getchar(f);
		if (ch==EOF) return word_new("&");
		if (ch!='#' && !special_valid_first_char(ch)) {
			my_unget (ch);
		} else {
			do {
				str[ix++]=ch; 
				if (ix==MAX_WORD_LEN) {
	str[ix]=0;
	fprintf (logfile,"1str=%s\n", str);
					error_handler ("encountered an extremely long word");
				}
				if (ch==';')break;
				ch=my_getchar(f);
			}
			  while(ch!=EOF);
		}
	}
	else {
		str[ix++]=ch;
		while (EOF!=(ch=my_getchar(f))) {
			if (ch=='&') {
				int ch2=my_getchar(f);
				if (ch2=='<' || ch2==' ' || special_valid_first_char(ch2)) {
					my_unget(ch2);
					break;
				} else {
					str[ix++]=ch;
					str[ix++]=ch2;
					str[ix]=0;
					continue;
				}
			}
			if (ch==' ' && break_at_whitespace) break;
			if (ch=='&'||ch=='<')break;
			str[ix++]=ch; 
/* fprintf (logfile,"%c",ch); */
			if (ix==MAX_WORD_LEN) {
str[ix]=0;
fprintf (logfile,"2str=%s\n", str);
				error_handler ("encountered an extremely long word");
			}
		}
		if(ch=='&'||ch=='<') my_unget(ch);
		if(ch==' ') { 
			following_whitespace=(1);
			while (ch==' ') ch = my_getchar(f);
			my_unget(ch);
		}
	}
	str[ix]=0;
	w=word_new(str);
	if (following_whitespace)
		w->flags |= WORD_HAS_FOLLOWING_WHITESPACE;

	return w;
}


/* This is a hash of the tags which are not usually
 * vulnerable to being incorrectly used.
 */

typedef struct {
	char *name;
	char *until_for_unget0;
	char *until_for_unget1;
	char *until_for_unget2;
	char *until_for_unget3;
}
TagParseData;

static TagParseData reliable_a[] = {
	{ "author", "/head", "body", "title", NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_b[] = {
	{ "b", NULL, NULL, NULL, NULL },
	{ "big", NULL, NULL, NULL, NULL },
	{ "body", "body", "/html", NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_c [] = {
	{ "center", NULL, NULL, NULL, NULL },
	{ "code", NULL, NULL, NULL, NULL },
	{ "colgroup", NULL, NULL, NULL, NULL },
	{ "cols", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_d [] = {
	{ "date", "date", NULL, NULL, NULL },
	{ "del", NULL, NULL, NULL, NULL },
	{ "dl", NULL, NULL, NULL, NULL },
	{ "draw", NULL, NULL, NULL, NULL },
	{ "dt", "dd", "/dl", NULL, NULL },
	{ "dd", "dt", "/dl", NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_e [] = {
	{ "em", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_f [] = {
	{ "font", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_h [] = {
	{ "h1", NULL, NULL, NULL, NULL },
	{ "h2", NULL, NULL, NULL, NULL },
	{ "h3", NULL, NULL, NULL, NULL },
	{ "h4", NULL, NULL, NULL, NULL },
	{ "h5", NULL, NULL, NULL, NULL },
	{ "h6", NULL, NULL, NULL, NULL },
	{ "head", "body", "/html", "head", NULL },
	{ "html", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_i [] = {
	{ "i", NULL, NULL, NULL, NULL },
	{ "ins", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_l [] = {
	{ "li", "li", "/ol", "/ul", "/menu"},
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_m [] = {
	{ "model", NULL, NULL, NULL, NULL },
	{ "menu", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_n [] = {
	{ "noscript", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_o [] = {
	{ "ol", NULL, NULL, NULL, NULL },
	{ "over", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL },
};

static TagParseData reliable_p [] = {
	{ "p", "/body", "/html", NULL, NULL },
	{ "pre", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_q [] = {
	{ "q", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_s [] = {
	{ "s", NULL, NULL, NULL, NULL },
	{ "script", NULL, NULL, NULL, NULL },
	{ "small", NULL, NULL, NULL, NULL },
	{ "span", NULL, NULL, NULL, NULL },
	{ "strike", NULL, NULL, NULL, NULL },
	{ "strong", NULL, NULL, NULL, NULL },
	{ "style", NULL, NULL, NULL, NULL },
	{ "sub", NULL, NULL, NULL, NULL },
	{ "sup", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_t [] = {
	{ "table", NULL, NULL, NULL, NULL },
	{ "tbody", NULL, NULL, NULL, NULL },
	{ "td", "td", "/tr", "tr", "th" },
	{ "tr", "tr", "/table", "tr", NULL },
	{ "th", "th", "/tr", "tr", "td" },
	{ "tt", NULL, NULL, NULL, NULL },
	{ "title", "/head", "body", "author", NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData reliable_u [] = {
	{ "u", NULL, NULL, NULL, NULL },
	{ "ul", NULL, NULL, NULL, NULL },
	{ NULL, NULL, NULL, NULL, NULL },
};

static TagParseData* reliable_tags [] = {
	reliable_a,
	reliable_b,
	reliable_c,
	reliable_d,
	reliable_e,
	reliable_f,
	NULL,	
	reliable_h,
	reliable_i,
	NULL,
	NULL,
	reliable_l,
	reliable_m,
	reliable_n,
	reliable_o,
	reliable_p,
	reliable_q,
	NULL,
	reliable_s,
	reliable_t,
	reliable_u,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
};


/* until_for_unget# is the tag which should result
 * in our ungetting the word as read, rather than
 * disposing of it. This string must have a '/'
 * if it will be needed. until is the tag that can
 * be ignored, and is assumed to have a '/'.
 */
Word* 
word_metaread(FILE *f, 
		String until, 
		String until_for_unget0, 
		String until_for_unget1, 
		String until_for_unget2,
		String until_for_unget3,
		int within_pre) 
{
	Word *w=NULL;
	Word *lastw=NULL;
	Word *w2;
	int ch;

	CHECK_PARAM_NOT_NULL(f);

	while((w2=word_read(f, within_pre ? FALSE : TRUE))) {
		ch=w2->str[0];

		/* Ignore whitespace when it begins a sequence of Words.
		 */
		if (ch==' ' && !w) {
			continue;
		}

		/* Example TD terminating with /TD */
		if(until && '<'==ch && '/'==w2->str[1]) {
			int len=strlen(until);
			if(!strncmp(&w2->str[2],until,len)) {
				int ch2=w2->str[len+2];
				if(!isdigit(ch2)&&!isalpha(ch2)) {
/* if (debug_mode && logfile) fprintf (logfile,"ACCEPTING candidate until string %s as matching %s\n", w2->str,until);  */
					word_free(w2);
					return w;
				} else {
/* if (debug_mode && logfile) fprintf (logfile,"REJECTING candidate until string %s as not matching %s\n", w2->str,until);  */
				}
			}
		}

		if(until_for_unget0 && '<'==ch) {
			int len=strlen(until_for_unget0);
			if(!strncmp(&w2->str[1],until_for_unget0,len)) {
				int ch2=w2->str[len+1];
				if(!isdigit(ch2)&&!isalpha(ch2)) {
/* if (debug_mode && logfile) fprintf (logfile,"ACCEPTING candidate until string %s as matching %s\n", w2->str,until); */
					word_unget (w2);
					return w;
				} else {
/* if (debug_mode && logfile) fprintf (logfile,"REJECTING candidate until string %s as not matching %s\n", w2->str,until); */
				}
			}
		}

		if(until_for_unget1 && '<'==ch) {
			int len=strlen(until_for_unget1);
			if(!strncmp(&w2->str[1],until_for_unget1,len)) {
				int ch2=w2->str[len+1];
				if(!isdigit(ch2)&&!isalpha(ch2)) {
/* if (debug_mode && logfile) fprintf (logfile,"ACCEPTING candidate until string %s as matching %s\n", w2->str,until); */
					word_unget (w2);
					return w;
				} else {
/* if (debug_mode && logfile) fprintf (logfile,"REJECTING candidate until string %s as not matching %s\n", w2->str,until); */
				}
			}
		}

		if(until_for_unget2 && '<'==ch) {
			int len=strlen(until_for_unget2);
			if(!strncmp(&w2->str[1],until_for_unget2,len)) {
				int ch2=w2->str[len+1];
				if(!isdigit(ch2)&&!isalpha(ch2)) {
/* if (debug_mode && logfile) fprintf (logfile,"ACCEPTING candidate until string %s as matching %s\n", w2->str,until); */
					word_unget (w2);
					return w;
				} else {
/* if (debug_mode && logfile) fprintf (logfile,"REJECTING candidate until string %s as not matching %s\n", w2->str,until); */
				}
			}
		}

		if(until_for_unget3 && '<'==ch) {
			int len=strlen(until_for_unget3);
			if(!strncmp(&w2->str[1],until_for_unget3,len)) {
				int ch2=w2->str[len+1];
				if(!isdigit(ch2)&&!isalpha(ch2)) {
/* if (debug_mode && logfile) fprintf (logfile,"ACCEPTING candidate until string %s as matching %s\n", w2->str,until);  */
					word_unget (w2);
					return w;
				} else {
/* if (debug_mode && logfile) fprintf (logfile,"REJECTING candidate until string %s as not matching %s\n", w2->str,until); */
				}
			}
		}

		if(!w) {
			w=lastw=w2;
		} else {
			lastw->next=w2; lastw=w2;
		}

		if('<'==ch) {
			String arg0;

			/* Break up the tag into more useful segments.
			 */
			if (!word_divide_tag (w2))
				error_handler ("cannot divide Word");

			arg0 = w2->segments->str;

			/* Fetch children if any, except for certain tags
			 * If not listed here, they are not known to have
			 * children.
			 */
			if (arg0[0]=='a' && !arg0[1]) {
				/* <a> which lacks an href does not have children.
				 * Otherwise it does and terms with </a>.
				 */
				if (strstr(w2->str,"href"))
					w2->child=word_metaread(f,"a",NULL,NULL,NULL,NULL,FALSE);
			} else {
				TagParseData *array;
				int ix;

				/* We shall now look in the hash of "reliable" 
				 * tags, i.e. those whose ending tags are not
				 * generally thought of as optional.
				 */
				ix = tolower(*arg0) - 'a';
				if (ix>=0 && ix<26) 
				{
					char *ptr;

					array=reliable_tags[ix];
					if (array) {
						ix = 0;
						while ((ptr = array[ix].name)) {
							if (!strcmp (ptr,arg0)) {
								int within_pre;
								if (!strcmp(arg0, "pre")) 
									within_pre = TRUE;
								else
									within_pre = FALSE;

								w2->child=word_metaread(f,arg0,
									array[ix].until_for_unget0,
									array[ix].until_for_unget1,
									array[ix].until_for_unget2,
									array[ix].until_for_unget3,
									within_pre);
								break;
							}
							ix++;
						}
					}
				}
			}
		}
	}

	/* Ignore whitespace when it ends a sequence of Words.
	 */
	if (lastw && lastw->str[0]==' ') {
		Word *x;

		if (w==lastw) {
			word_free (w);
			return NULL;
		}

		x=w;
		while (x && x->next) {
			if (x->next == lastw) {
				x->next=NULL;
				word_free (lastw);
			}
			x=x->next;
		}
	}

	return w;
}


static int level=0;
void word_dump(FILE *f, Word *w) {
	int i;
	CHECK_PARAM_NOT_NULL(w);
	if(level) for(i=0;i<level;i++) fprintf(f," ");
	while(w) {
		if (*w->str == '<') {
			Segment *s = w->segments;
			fprintf (f,"TAG: ");
			while(s) {
				fprintf (f,"'%s', ", s->str);
				s=s->next;
			}
		fprintf (f,"\n... @ %ld,%ld  dims=(%ld x [%lu+%lu])\n",
			w->x, w->y, w->width, w->ascent, w->descent);
		}
		else
		fprintf (f,"WORD: \"%s\"  @ %ld,%ld  dims=(%ld by [%lu+%lu])\n",w->str,
			w->x, w->y, w->width, w->ascent, w->descent);

		if(w->child) { 
			if(w && level) for(i=0;i<level;i++) fprintf(f," ");
			fprintf(f,"{\n"); 
			level+=2;
			word_dump(f,w->child); 
			level-=2;
			if(w && level) for(i=0;i<level;i++) fprintf(f," ");
			fprintf(f,"}\n"); 
		}
		w=w->next;
		if(w && level) for(i=0;i<level;i++) fprintf(f," ");
	}
}


Word*
word_parse (FILE *f)
{
	return word_metaread (f, NULL,NULL,NULL,NULL,NULL, FALSE);
}
