
/*=============================================================================
   beest, an HTML viewer.
   Copyright (C) 2000 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at zeyr@excite.com.
=============================================================================*/

/*----------------------------------------------------------------------
 * Module name:    table.h
 * Author name:    Zach Smith
 * Create date:    1 Oct 2000
 * Last change:    13 Dec 2000
 * Purpose:        Type definitions, prototypes, defines for Table class.
 *----------------------------------------------------------------------
 * Changes:
 *--------------------------------------------------------------------*/

#ifndef _TABLE
#define _TABLE

#ifndef _DISPLAY
#include "display.h"
#endif

#ifndef _WORD
#include "word.h"
#endif

#ifndef _GROUP
#include "group.h"
#endif

typedef struct {
	Dimension optimal_width;
	Dimension minimal_width;
	Dimension desired_width; /* of the Group's max_width */

	Dimension optimal_height;
	Dimension desired_height;

	unsigned short col_span;
	unsigned short row_span;

	unsigned char is_leading_cell; /* false=>is already part of big cell */

	Dimension box_width, box_height;

	Word *td;
}
CellInfo;


typedef struct 
{
	Word *word; /* the table tag word */
	
	Coordinate x0, y0, x1, y1;

	/* Number of rows and columns.
	 */
	int min_columns, columns, rows;

	/* HTML-defined parameters.
	 */
	Dimension cell_spacing, cell_padding;
	Dimension border_size;
	Color border_color;

	/* Overall table dimensions
	 */
	CellInfo overall;

	/* Allocated after table row/column counts determined ...
	 */
	CellInfo *cells;

	/* Used to assign percentage-of-width values.
	 */
	Dimension max_content_width;

	/* Row and column dimensions
	 */
	Dimension *row_heights;
	Dimension *column_widths;
}
Table;

extern Table *table_new (void);
extern void table_free (Table*);
extern void table_dump (FILE *f, Table*);
extern void table_redraw (DisplayDevice*, Table*,
	Coordinate,Coordinate,
	Coordinate,Coordinate,
	Coordinate,Coordinate,
	Coordinate,Coordinate);

#endif

